#include <opencv2/opencv.hpp>
#include <opencv2/ml.hpp>
#include <opencv2/xfeatures2d.hpp>
#include <opencv2/features2d/features2d.hpp>

#include <iostream>
#include <fstream>

#include "Svm.h"

using namespace std;
using namespace cv;
using namespace cv::ml;

#define num_train_sample  200
#define num_test_sample   200


#define with_steps
#define feature_size 1152 //para hog

void extractFeaturesHOG(Mat &train, Mat &test, Mat &train_l, Mat &test_l);
void readLabels(Mat &train_l, Mat &test_l);


int main(int argc, char** argv) {

    //matrizes para armazenar features
    Mat matTrainFeatures = Mat::zeros(num_train_sample,feature_size, CV_32F);
    Mat matTestFeatures  = Mat::zeros(num_test_sample,feature_size,CV_32F);
    
    //matrizes para armazenar labels
    Mat matTrainLabels= Mat::zeros(1,num_train_sample,CV_32SC1);
    Mat matTestLabels= Mat::zeros(1,num_test_sample,CV_32SC1);

    //armazena os resultados
    //0,0 diz que será alocada pela quantidade de testes que existir
    Mat matResults(0,0,CV_32SC1);
    
    //le as imagens
    //converte em linhas
#ifdef with_steps
    cout << "Carregando features ..." << endl;
#endif    
    extractFeaturesHOG(matTrainFeatures, matTestFeatures, matTrainLabels, matTestLabels);
    
    //depois le os labels
#ifdef with_steps
    cout << "Carregando labels ..." << endl;
#endif    
    readLabels(matTrainLabels, matTestLabels);
    
    //instacia e usa svm
    Svm svm;
    
#ifdef with_steps
    cout << "Treinando SVM ..." << endl;
#endif
    svm.loadTrainAndAdjust(matTrainFeatures, matTrainLabels);
    //se tiver os parâmetros, use svm.loadAndTrain(matTrainFeatures, matTrainLabels, C, gamma);
    
#ifdef with_steps
    cout << "Parâmetros SVM -> C: " << svm.c << " gamma: " << svm.gamma << endl;
#endif    
    svm.predict(matTestFeatures);

#ifdef with_steps
    cout << "Gerando Resultados ..." << endl;
#endif
    svm.checkResults(matTestLabels, 10);

    return 0;
}


void extractFeaturesHOG(Mat &train, Mat &test, Mat &train_l, Mat &test_l) {
    ostringstream nome;
    int i,k, desc;
    
    HOGDescriptor hog( Size(16,16), Size(8,8), Size(4,4), Size(4,4), 8); 
    vector<float> descriptorsValues;
    
    for (i=1; i<=num_train_sample; i++){
        nome.str("");
        nome << "train/" << i << ".png";
        Mat im = imread(nome.str(),0);
        
        //gera HOG
        k=0;
        hog.compute( im, descriptorsValues);
       
        for (desc=0; desc<descriptorsValues.size(); desc++) 
            train.at<float>(i-1,k++) = descriptorsValues.at(desc);
    }
    
    
    
    
    for (i=6001; i<=6000+num_test_sample; i++){
        nome.str("");
        nome << "test/" << i << ".png";
        Mat im = imread(nome.str(),0);
        
       //gera HOG
        k=0;
        hog.compute( im, descriptorsValues);
        
        for (desc=0; desc<descriptorsValues.size(); desc++) 
            test.at<float>(i-6001,k++) = descriptorsValues.at(desc);
    }
    
    readLabels(train_l, test_l);

}

void readLabels(Mat &train_l, Mat &test_l) {
    
    ifstream labels ("labels_bm.txt");
    int elm;
    if (labels.is_open()) 
        
    
        for (int i=0; i<6000; i++) {
            labels >> elm;
            if (i <num_train_sample)
                train_l.at<int>(0,i) = elm;
        }
        for (int i=0; i<6000; i++) {
            labels >> elm;
            if (i < num_test_sample)
                test_l.at<int>(0,i) = elm;
        }
    
    labels.close();
}